import { BaseEvent } from '#application/base/baseEvent'
import ZoneManager from '#managers/zoneManager'
import CharacterRepository from '#repositories/characterRepository'

export default class ZoneLeaveEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('zone:character:leave', this.handleEvent.bind(this))
  }

  private async handleEvent(): Promise<void> {
    try {
      if (!this.socket.characterId) {
        this.logger.error('zone:character:leave error: Zone requested but no character id set')
        return
      }

      const character = await CharacterRepository.getById(this.socket.characterId)
      if (!character) {
        this.logger.error('zone:character:leave error: Character not found')
        return
      }

      /**
       * @TODO: If zone is not found, spawn back to the start
       */
      const zone = character.zone
      if (!zone) {
        this.logger.error('zone:character:leave error: Zone not found')
        return
      }

      const loadedZone = ZoneManager.getZoneById(zone.id)
      if (!loadedZone) {
        this.logger.error('zone:character:leave error: Loaded zone not found')
        return
      }

      this.socket.leave(zone.id.toString())

      // let other clients know of character leaving
      this.io.to(zone.id.toString()).emit('zone:character:leave', character.id)

      // remove character from zone manager
      await loadedZone.removeCharacter(character.id)

      this.logger.info('zone:character:leave ' + `Character ${character.id} left zone ${zone.id}`)
    } catch (error: any) {
      this.logger.error('zone:character:leave error: ' + error.message)
    }
  }
}
